-- Copyright (c) 2006 Frank Buss (fb@frank-buss.de)
-- See license.txt for license
--
-- See readme.html for documentation

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity test is
  port(
    ds_wire: inout std_logic;
    clk_50mhz: in std_logic;
    rs232_dce_txd: out std_logic;
    rs232_dce_rxd: in std_logic;
    led: out unsigned(7 downto 0));
end entity test;

architecture rtl of test is
  constant system_speed: natural := 50e6;
  constant baudrate: natural := 115200;

  signal rs232_receiver_ack: std_logic := '0';
  signal rs232_receiver_dat: unsigned(7 downto 0) := (others => '0');
  signal rs232_receiver_stb: std_logic := '0';

  signal rs232_sender_ack: std_logic := '0';
  signal rs232_sender_dat: unsigned(7 downto 0);
  signal rs232_sender_stb: std_logic := '0';

  signal led_latch: unsigned(7 downto 0) := (others => '0');
  signal counter: natural range 0 to (system_speed / 2) := 0;
  
  signal ds_2432_rom: unsigned(47 downto 0) := (others => '0');
  signal ds_2432_rom_latch: unsigned(47 downto 0) := (others => '0');
  signal ds_2432_rom_valid: std_logic := '0';
  signal ds_2432_rom_nibble_counter: natural range 0 to 12 := 0;
  
  type state_type is (
    wait_for_command,
    wait_for_strobe,
    send_data,
    wait_for_data_send_start);

  signal state: state_type := wait_for_command;
  signal echo: std_logic;
  
begin

  sender: entity rs232_sender
    generic map(system_speed, baudrate)
    port map(
      ack_o => rs232_sender_ack,
      clk_i => clk_50mhz,
      dat_i => rs232_sender_dat,
      rst_i => '0',
      stb_i => rs232_sender_stb,
      tx => rs232_dce_txd);

  receiver: entity rs232_receiver
    generic map(system_speed, baudrate)
    port map(
      ack_i => rs232_receiver_ack,
      clk_i => clk_50mhz,
      dat_o => rs232_receiver_dat,
      rst_i => '0',
      stb_o => rs232_receiver_stb,
      rx => rs232_dce_rxd);

  ds2432_inst: entity ds2432
    generic map(system_speed)
    port map(
      clk_i => clk_50mhz,
      rst_i => '0',
      rom_id_o => ds_2432_rom,
      data_valid_o => ds_2432_rom_valid,
      wire_io => ds_wire);

  ds_2432_rom_dump: process(clk_50mhz)
    variable digit: natural range 0 to 15;
  begin
    if rising_edge(clk_50mhz) then
      case state is
        -- read command from RS232 port
        when wait_for_command =>
          if rs232_receiver_stb = '1' then
            state <= wait_for_strobe;
            rs232_receiver_ack <= '1';
          end if;
        when wait_for_strobe =>
          if rs232_receiver_stb <= '0' then
            rs232_receiver_ack <= '0';
            state <= send_data;
            echo <= '1';
          end if;

        -- send echo and DS2432 unique ROM id to RS232 port
        when send_data =>
          if rs232_sender_ack = '0' then
            if echo = '1' then
              rs232_sender_stb <= '1';
              rs232_sender_dat <= rs232_receiver_dat;
              rs232_sender_stb <= '1';
              state <= wait_for_data_send_start;
            else
              if ds_2432_rom_nibble_counter = 0 then
                state <= wait_for_command;
              else
                digit := to_integer(ds_2432_rom_latch(47 downto 44));
                if digit < 10 then
                  rs232_sender_dat <= digit + x"30";
                else
                  rs232_sender_dat <= digit + x"57";
                end if;
                rs232_sender_stb <= '1';
                state <= wait_for_data_send_start;
              end if;
            end if;
          end if;
        when wait_for_data_send_start =>
          if rs232_sender_ack = '1' then
            rs232_sender_stb <= '0';
            if echo = '1' then
              echo <= '0';
              state <= wait_for_command;
              case rs232_receiver_dat is
                when x"69" =>  -- "i": identify
                  if ds_2432_rom_valid = '1' then
                    ds_2432_rom_nibble_counter <= 12;
                    ds_2432_rom_latch <= ds_2432_rom;
                    state <= send_data;
                  end if;
                when x"30" =>  -- "0": LEDs off
                  led_latch(7 downto 1) <= (others => '0');
                when x"31" =>  -- "1": LEDs on
                  led_latch(7 downto 1) <= (others => '1');
                when others =>
              end case;
            else
              ds_2432_rom_latch <= shift_left(ds_2432_rom_latch, 4);
              ds_2432_rom_nibble_counter <= ds_2432_rom_nibble_counter - 1;
              state <= send_data;
            end if;
          end if;
      end case;
  
      -- "test" should change with 1 Hz
      if counter = 0 then
        led_latch(0) <= not led_latch(0);
        counter <= system_speed / 2;
      else
        counter <= counter - 1;
      end if;
    end if;
  end process;

  led <= led_latch;
  
end architecture rtl;
